@@ -1,13 +1,19 @@
-# $Id: Ciphertext.pm,v 1.13 2001/07/29 06:29:52 btrott Exp $
+# $Id: Ciphertext.pm,v 1.15 2001/08/09 18:57:35 btrott Exp $
 
 package Crypt::OpenPGP::Ciphertext;
 use strict;
 
 use Crypt::OpenPGP::Cipher;
-use Crypt::OpenPGP::Constants qw( DEFAULT_CIPHER );
+use Crypt::OpenPGP::Constants qw( DEFAULT_CIPHER
+                                  PGP_PKT_ENCRYPTED
+                                  PGP_PKT_ENCRYPTED_MDC );
 use Crypt::OpenPGP::ErrorHandler;
 use base qw( Crypt::OpenPGP::ErrorHandler );
 
+use constant MDC_TRAILER => chr(0xd3) . chr(0x14);
+
+sub pkt_type { $_[0]->{is_mdc} ? PGP_PKT_ENCRYPTED_MDC : PGP_PKT_ENCRYPTED }
+
 sub new {
     my $class = shift;
     my $enc = bless { }, $class;
@@ -18,6 +24,8 @@ sub init {
     my $enc = shift;
     my %param = @_;
     if ((my $key = $param{SymKey}) && (my $data = $param{Data})) {
+        $enc->{is_mdc} = $param{MDC} || 0;
+        $enc->{version} = 1;
         require Crypt::Random;
         my $alg = $param{Cipher} || DEFAULT_CIPHER;
         my $cipher = Crypt::OpenPGP::Cipher->new($alg, $key);
@@ -25,21 +33,41 @@ sub init {
         my $pad = Crypt::Random::makerandom_octet( Length => $bs );
         $pad .= substr $pad, -2, 2;
         $enc->{ciphertext} = $cipher->encrypt($pad);
-        $cipher->decrypt(substr $enc->{ciphertext}, 2, $bs);   ## resync
+        $cipher->sync unless $enc->{is_mdc};
         $enc->{ciphertext} .= $cipher->encrypt($data);
+
+        if ($enc->{is_mdc}) {
+            require Crypt::OpenPGP::MDC;
+            my $mdc = Crypt::OpenPGP::MDC->new(
+                Data => $pad . $data . MDC_TRAILER );
+            my $mdc_buf = Crypt::OpenPGP::PacketFactory->save($mdc);
+            $enc->{ciphertext} .= $cipher->encrypt($mdc_buf);
+        }
     }
     $enc;
 }
 
 sub parse {
     my $class = shift;
-    my($buf) = @_;
+    my($buf, $is_mdc) = @_;
     my $enc = $class->new;
-    $enc->{ciphertext} = $buf->get_bytes($buf->length);
+    $enc->{is_mdc} = $is_mdc;
+    if ($is_mdc) {
+        $enc->{version} = $buf->get_int8;
+    }
+    $enc->{ciphertext} = $buf->get_bytes($buf->length - $buf->offset);
     $enc;
 }
 
-sub save { $_[0]->{ciphertext} }
+sub save {
+    my $enc = shift;
+    my $buf = Crypt::OpenPGP::Buffer->new;
+    if ($enc->{is_mdc}) {
+        $buf->put_int8($enc->{version});
+    }
+    $buf->put_bytes($enc->{ciphertext});
+    $buf->bytes;
+}
 
 sub decrypt {
     my $enc = shift;
@@ -47,10 +75,25 @@ sub decrypt {
     my $cipher = Crypt::OpenPGP::Cipher->new($sym_alg, $key) or
         return $enc->error( Crypt::OpenPGP::Cipher->errstr );
     my $padlen = $cipher->blocksize + 2;
-    my $pt = $cipher->decrypt(substr $enc->{ciphertext}, 0, $padlen);
+    my $pt = $enc->{prefix} =
+        $cipher->decrypt(substr $enc->{ciphertext}, 0, $padlen);
     return $enc->error("Bad checksum")
         unless substr($pt, -4, 2) eq substr($pt, -2, 2);
-    $cipher->decrypt(substr $enc->{ciphertext}, $padlen);
+    $cipher->sync unless $enc->{is_mdc};
+    $pt = $cipher->decrypt(substr $enc->{ciphertext}, $padlen);
+    if ($enc->{is_mdc}) {
+        my $mdc_buf = Crypt::OpenPGP::Buffer->new_with_init(substr $pt,-22,22);
+        $pt = substr $pt, 0, -22;
+        my $mdc = Crypt::OpenPGP::PacketFactory->parse($mdc_buf);
+        return $enc->error("Encrypted MDC packet without MDC")
+            unless $mdc && ref($mdc) eq 'Crypt::OpenPGP::MDC';
+        require Crypt::OpenPGP::Digest;
+        my $dgst = Crypt::OpenPGP::Digest->new('SHA1');
+        my $hash = $dgst->hash($enc->{prefix} . $pt . chr(0xd3) . chr(0x14));
+        return $enc->error("SHA-1 hash of plaintext does not match MDC body")
+            unless $mdc->digest eq $hash;
+    }
+    $pt;
 }
 
 1;
@@ -78,10 +121,13 @@ Crypt::OpenPGP::Ciphertext - Encrypted data packet
 =head1 DESCRIPTION
 
 I<Crypt::OpenPGP::Ciphertext> implements symmetrically encrypted data
-packets, providing both encryption and decryption functionality. The
-encryption used in these packets is described in the OpenPGP RFC, in
-section 12.8 (OpenPGP CFB mode). It is a variant in standard CFB, and
-it differs slightly from the CFB used to encrypt secret key data.
+packets, providing both encryption and decryption functionality. Both
+standard encrypted data packets and encrypted-MDC (modification
+detection code) packets are supported by this class. In the first case,
+the encryption used in the packets is a variant on standard CFB mode,
+and is described in the OpenPGP RFC, in section 12.8 (OpenPGP CFB mode).
+In the second case (encrypted-MDC packets), the encryption is performed
+in standard CFB mode, without the special resync used in PGP's CFB.
 
 =head1 USAGE
 
@@ -121,6 +167,18 @@ for a list of valid cipher names.
 This argument is optional; by default I<Crypt::OpenPGP::Cipher> will
 use C<DES3>.
 
+=item * MDC
+
+When set to a true value, encrypted texts will use encrypted MDC
+(modification detection code) packets instead of standard encrypted data
+packets. These are a newer form of encrypted data packets that
+are followed by a C<SHA-1> hash of the plaintext data. This prevents
+attacks that modify the encrypted text by using a message digest to
+detect changes.
+
+By default I<MDC> is set to C<0>, and encrypted texts use standard
+encrypted data packets. Set it to a true value to turn on MDC packets.
+
 =back
 
 =head2 $ct->save
